@@ -19,12 +19,11 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 # ======================================================================
 
-from __future__ import absolute_import
-
 from .eam import EAM
 from .pair_potential import PairPotential
 from .supercell_calculator import SupercellCalculator
 from .polydisperse import Polydisperse
+from .manybody import Manybody
 
 try:
     import scipy.sparse as sp
