@@ -23,20 +23,24 @@
 Simple pair potential.
 """
 
-from __future__ import division
-
-import os
-
-import sys
-
-import time
+#
+# Coding convention
+# * All numpy arrays are suffixed with the array dimensions
+# * The suffix stands for a certain type of dimension:
+#   - n: Atomic index, i.e. array dimension of length nb_atoms
+#   - p: Pair index, i.e. array dimension of length nb_pairs
+#   - c: Cartesian index, array dimension of length 3
+#
 
 import numpy as np
 
-import ase
-from ase.calculators.calculator import Calculator
+from scipy.sparse import bsr_matrix, vstack, hstack
 
-from matscipy.neighbours import neighbour_list, first_neighbours
+import ase
+
+from ...neighbours import neighbour_list, first_neighbours
+from ..calculator import MatscipyCalculator
+from ...numpy_tricks import mabincount
 
 
 ###
@@ -145,7 +149,7 @@ class LennardJonesLinear():
         self.offset_force = 6/cutoff * \
             (-2*(sigma/cutoff)**12+(sigma/cutoff)**6)
 
-    def get_cutoff():
+    def get_cutoff(self):
         return self.cutoff
 
     def __call__(self, r):
@@ -266,13 +270,13 @@ class LennardJones84():
 ###
 
 
-class PairPotential(Calculator):
-    implemented_properties = ['energy', 'stress', 'forces', "hessian"]
+class PairPotential(MatscipyCalculator):
+    implemented_properties = ['energy', 'free_energy', 'stress', 'forces', 'hessian']
     default_parameters = {}
     name = 'PairPotential'
 
     def __init__(self, f, cutoff=None):
-        Calculator.__init__(self)
+        MatscipyCalculator.__init__(self)
         self.f = f
 
         self.dict = {x: obj.get_cutoff() for x, obj in f.items()}
@@ -280,263 +284,181 @@ class PairPotential(Calculator):
         self.df2 = {x: obj.derivative(2) for x, obj in f.items()}
 
     def calculate(self, atoms, properties, system_changes):
-        Calculator.calculate(self, atoms, properties, system_changes)
+        MatscipyCalculator.calculate(self, atoms, properties, system_changes)
 
-        nat = len(self.atoms)
+        nb_atoms = len(self.atoms)
         atnums = self.atoms.numbers
         atnums_in_system = set(atnums)
 
-        i_n, j_n, dr_nc, abs_dr_n = neighbour_list(
-            'ijDd', self.atoms, self.dict)
+        i_p, j_p, r_p, r_pc = neighbour_list('ijdD', self.atoms, self.dict)
 
-        e_n = np.zeros_like(abs_dr_n)
-        de_n = np.zeros_like(abs_dr_n)
+        e_p = np.zeros_like(r_p)
+        de_p = np.zeros_like(r_p)
         for params, pair in enumerate(self.dict):
             if pair[0] == pair[1]:
-                mask1 = atnums[i_n] == pair[0]
-                mask2 = atnums[j_n] == pair[0]
+                mask1 = atnums[i_p] == pair[0]
+                mask2 = atnums[j_p] == pair[0]
                 mask = np.logical_and(mask1, mask2)
 
-                e_n[mask] = self.f[pair](abs_dr_n[mask])
-                de_n[mask] = self.df[pair](abs_dr_n[mask])
+                e_p[mask] = self.f[pair](r_p[mask])
+                de_p[mask] = self.df[pair](r_p[mask])
 
             if pair[0] != pair[1]:
                 mask1 = np.logical_and(
-                    atnums[i_n] == pair[0], atnums[j_n] == pair[1])
+                    atnums[i_p] == pair[0], atnums[j_p] == pair[1])
                 mask2 = np.logical_and(
-                    atnums[i_n] == pair[1], atnums[j_n] == pair[0])
+                    atnums[i_p] == pair[1], atnums[j_p] == pair[0])
                 mask = np.logical_or(mask1, mask2)
 
-                e_n[mask] = self.f[pair](abs_dr_n[mask])
-                de_n[mask] = self.df[pair](abs_dr_n[mask])
+                e_p[mask] = self.f[pair](r_p[mask])
+                de_p[mask] = self.df[pair](r_p[mask])
 
-        epot = 0.5*np.sum(e_n)
+        epot = 0.5*np.sum(e_p)
 
         # Forces
-        df_nc = -0.5*de_n.reshape(-1, 1)*dr_nc/abs_dr_n.reshape(-1, 1)
+        df_pc = -0.5*de_p.reshape(-1, 1)*r_pc/r_p.reshape(-1, 1)
 
-        # Sum for each atom
-        fx_i = np.bincount(j_n, weights=df_nc[:, 0], minlength=nat) - \
-            np.bincount(i_n, weights=df_nc[:, 0], minlength=nat)
-        fy_i = np.bincount(j_n, weights=df_nc[:, 1], minlength=nat) - \
-            np.bincount(i_n, weights=df_nc[:, 1], minlength=nat)
-        fz_i = np.bincount(j_n, weights=df_nc[:, 2], minlength=nat) - \
-            np.bincount(i_n, weights=df_nc[:, 2], minlength=nat)
+        f_nc = mabincount(j_p, df_pc, nb_atoms) - mabincount(i_p, df_pc, nb_atoms)
 
         # Virial
-        virial_v = -np.array([dr_nc[:, 0]*df_nc[:, 0],               # xx
-                              dr_nc[:, 1]*df_nc[:, 1],               # yy
-                              dr_nc[:, 2]*df_nc[:, 2],               # zz
-                              dr_nc[:, 1]*df_nc[:, 2],               # yz
-                              dr_nc[:, 0]*df_nc[:, 2],               # xz
-                              dr_nc[:, 0]*df_nc[:, 1]]).sum(axis=1)  # xy
+        virial_v = -np.array([r_pc[:, 0] * df_pc[:, 0],               # xx
+                              r_pc[:, 1] * df_pc[:, 1],               # yy
+                              r_pc[:, 2] * df_pc[:, 2],               # zz
+                              r_pc[:, 1] * df_pc[:, 2],               # yz
+                              r_pc[:, 0] * df_pc[:, 2],               # xz
+                              r_pc[:, 0] * df_pc[:, 1]]).sum(axis=1)  # xy
 
         self.results = {'energy': epot,
+                        'free_energy': epot,
                         'stress': virial_v/self.atoms.get_volume(),
-                        'forces': np.transpose([fx_i, fy_i, fz_i])}
+                        'forces': f_nc}
 
     ###
 
-    def calculate_hessian_matrix(self, atoms, H_format="dense", limits=None):
+    def get_hessian(self, atoms, format='dense', divide_by_masses=False):
         """
         Calculate the Hessian matrix for a pair potential.
         For an atomic configuration with N atoms in d dimensions the hessian matrix is a symmetric, hermitian matrix
-        with a shape of (d*N,d*N). The matrix is in general a sparse matrix, which consists of dense blocks of shape (d,d), which
-        are the mixed second derivatives. The result of the derivation for a pair potential can be found in:
-        L. Pastewka et. al. "Seamless elastic boundaries for atomistic calculations", Phys. Ev. B 86, 075459 (2012).
+        with a shape of (d*N,d*N). The matrix is in general a sparse matrix, which consists of dense blocks of
+        shape (d,d), which are the mixed second derivatives. The result of the derivation for a pair potential can be
+        found e.g. in:
+        L. Pastewka et. al. "Seamless elastic boundaries for atomistic calculations", Phys. Rev. B 86, 075459 (2012).
 
         Parameters
         ----------
         atoms: ase.Atoms
             Atomic configuration in a local or global minima.
 
-        H_format: "dense" or "sparse"
+        format: "sparse" or "neighbour-list"
             Output format of the hessian matrix.
-            The format "sparse" is only possible if matscipy was build with scipy.
 
-        limits: list [atomID_low, atomID_up]
-            Calculate the Hessian matrix only for the given atom IDs. 
-            If limits=[5,10] the Hessian matrix is computed for atom IDs 5,6,7,8,9 only.
-            The Hessian matrix will have the full shape dim(3*N,3*N) where N is the number of atoms. 
-            This ensures correct indexing of the data. 
+        divide_by_masses: bool
+            if true return the dynamic matrix else hessian matrix 
 
         Restrictions
         ----------
         This method is currently only implemented for three dimensional systems
-
         """
-
-        if H_format == "sparse":
-            try:
-                from scipy.sparse import bsr_matrix, vstack, hstack
-            except ImportError:
-                raise ImportError(
-                    "Import error: Can not output the hessian matrix since scipy.sparse could not be loaded!")
+        if self.atoms is None:
+            self.atoms = atoms
 
         f = self.f
         dict = self.dict
         df = self.df
         df2 = self.df2
 
-        nat = len(atoms)
+        nb_atoms = len(atoms)
         atnums = atoms.numbers
 
-        i_n, j_n, dr_nc, abs_dr_n = neighbour_list('ijDd', atoms, dict)
-        first_i = first_neighbours(nat, i_n)
+        i_p, j_p,  r_p, r_pc = neighbour_list('ijdD', atoms, dict)
+        first_i = first_neighbours(nb_atoms, i_p)
 
-        e_n = np.zeros_like(abs_dr_n)
-        de_n = np.zeros_like(abs_dr_n)
-        dde_n = np.zeros_like(abs_dr_n)
+        e_p = np.zeros_like(r_p)
+        de_p = np.zeros_like(r_p)
+        dde_p = np.zeros_like(r_p)
         for params, pair in enumerate(dict):
             if pair[0] == pair[1]:
-                mask1 = atnums[i_n] == pair[0]
-                mask2 = atnums[j_n] == pair[0]
+                mask1 = atnums[i_p] == pair[0]
+                mask2 = atnums[j_p] == pair[0]
                 mask = np.logical_and(mask1, mask2)
 
-                e_n[mask] = f[pair](abs_dr_n[mask])
-                de_n[mask] = df[pair](abs_dr_n[mask])
-                dde_n[mask] = df2[pair](abs_dr_n[mask])
+                e_p[mask] = f[pair](r_p[mask])
+                de_p[mask] = df[pair](r_p[mask])
+                dde_p[mask] = df2[pair](r_p[mask])
 
             if pair[0] != pair[1]:
                 mask1 = np.logical_and(
-                    atnums[i_n] == pair[0], atnums[j_n] == pair[1])
+                    atnums[i_p] == pair[0], atnums[j_p] == pair[1])
                 mask2 = np.logical_and(
-                    atnums[i_n] == pair[1], atnums[j_n] == pair[0])
+                    atnums[i_p] == pair[1], atnums[j_p] == pair[0])
                 mask = np.logical_or(mask1, mask2)
 
-                e_n[mask] = f[pair](abs_dr_n[mask])
-                de_n[mask] = df[pair](abs_dr_n[mask])
-                dde_n[mask] = df2[pair](abs_dr_n[mask])
-
-        if limits != None:
-            if limits[1] < limits[0]:
-                raise ValueError(
-                    "Value error: The upper atom id cannot be smaller than the lower atom id.")
-            else:
-                mask = np.logical_and(i_n >= limits[0], i_n < limits[1])
-                i_n = i_n[mask]
-                i_n1 = i_n - i_n[0]
-                j_n = j_n[mask]
-                dr_nc = dr_nc[mask]
-                abs_dr_n = abs_dr_n[mask]
-                e_n = e_n[mask]
-                de_n = de_n[mask]
-                dde_n = dde_n[mask]
-                nat1 = limits[1] - limits[0]
-
-                first_i = [0] * (nat1 + 1)
-                j = 1
-                for k in range(1, len(i_n)):
-                    if i_n[k] != i_n[k-1]:
-                        first_i[j] = k
-                        j = j+1
-                first_i[-1] = len(i_n)
-
-                if H_format == "sparse":
-                    # Off-diagonal elements of the Hessian matrix
-                    e_nc = (dr_nc.T / abs_dr_n).T
-                    H_ncc = -(dde_n * (e_nc.reshape(-1, 3, 1)
-                                       * e_nc.reshape(-1, 1, 3)).T).T
-                    H_ncc += -(de_n / abs_dr_n * (np.eye(3, dtype=e_nc.dtype)
-                                                  - (e_nc.reshape(-1, 3, 1) * e_nc.reshape(-1, 1, 3))).T).T
-
-                    H_nat1nat = bsr_matrix(
-                        (H_ncc, j_n, first_i), shape=(3*nat1, 3*nat))
-
-                    # Stack matrices in order to obtain full shape (3*nat, 3*nat)
-                    H = vstack([bsr_matrix((limits[0]*3, 3*nat)), H_nat1nat,
-                                bsr_matrix((3*nat - limits[1]*3, 3*nat))])
-
-                    # Diagonal elements of the Hessian matrix
-                    Hdiag_icc = np.empty((nat1, 3, 3))
-                    for x in range(3):
-                        for y in range(3):
-                            Hdiag_icc[:, x, y] = - \
-                                np.bincount(i_n1, weights=H_ncc[:, x, y])
-
-                    Hdiag_nat1nat = bsr_matrix((Hdiag_icc, np.arange(limits[0], limits[1]),
-                                                np.arange(nat1+1)), shape=(3*nat1, 3*nat))
-
-                    # Compute full Hessian matrix
-                    H += vstack([bsr_matrix((limits[0]*3, 3*nat)), Hdiag_nat1nat,
-                                 bsr_matrix((3*nat - limits[1]*3, 3*nat))])
-
-                    return H
-
-                elif H_format == "dense":
-                    # Off-diagonal elements of the Hessian matrix
-                    e_nc = (dr_nc.T / abs_dr_n).T
-                    H_ncc = -(dde_n * (e_nc.reshape(-1, 3, 1) *
-                                       e_nc.reshape(-1, 1, 3)).T).T
-                    H_ncc += -(de_n/abs_dr_n * (np.eye(3, dtype=e_nc.dtype)
-                                                - (e_nc.reshape(-1, 3, 1) * e_nc.reshape(-1, 1, 3))).T).T
-
-                    H = np.zeros((3*nat, 3*nat))
-                    for atom in range(len(i_n)):
-                        H[3*i_n[atom]:3*i_n[atom]+3,
-                          3*j_n[atom]:3*j_n[atom]+3] += H_ncc[atom]
-
-                    # Diagonal elements of the Hessian matrix
-                    Hdiag_icc = np.empty((nat1, 3, 3))
-                    for x in range(3):
-                        for y in range(3):
-                            Hdiag_icc[:, x, y] = - \
-                                np.bincount(i_n1, weights=H_ncc[:, x, y])
-
-                    Hdiag_ncc = np.zeros((3*nat, 3*nat))
-                    for atom in range(nat1):
-                        Hdiag_ncc[3*(atom+limits[0]):3*(atom+limits[0])+3,
-                                  3*(atom+limits[0]):3*(atom+limits[0])+3] += Hdiag_icc[atom]
-
-                    # Compute full Hessian matrix
-                    H += Hdiag_ncc
-
-                    return H
+                e_p[mask] = f[pair](r_p[mask])
+                de_p[mask] = df[pair](r_p[mask])
+                dde_p[mask] = df2[pair](r_p[mask])
+        
+        n_pc = (r_pc.T/r_p).T
+        H_pcc = -(dde_p * (n_pc.reshape(-1, 3, 1)
+                           * n_pc.reshape(-1, 1, 3)).T).T
+        H_pcc += -(de_p/r_p * (np.eye(3, dtype=n_pc.dtype)
+                                    - (n_pc.reshape(-1, 3, 1) * n_pc.reshape(-1, 1, 3))).T).T
 
         # Sparse BSR-matrix
-        elif H_format == "sparse":
-            e_nc = (dr_nc.T/abs_dr_n).T
-            H_ncc = -(dde_n * (e_nc.reshape(-1, 3, 1)
-                               * e_nc.reshape(-1, 1, 3)).T).T
-            H_ncc += -(de_n/abs_dr_n * (np.eye(3, dtype=e_nc.dtype)
-                                        - (e_nc.reshape(-1, 3, 1) * e_nc.reshape(-1, 1, 3))).T).T
+        if format == "sparse":
+            if divide_by_masses:
+                masses_n = self.atoms.get_masses()
+                geom_mean_mass_p = np.sqrt(masses_n[i_p]*masses_n[j_p])
 
-            H = bsr_matrix((H_ncc, j_n, first_i), shape=(3*nat, 3*nat))
+            if divide_by_masses:
+                H = bsr_matrix(((H_pcc.T/geom_mean_mass_p).T, j_p, first_i), shape=(3*nb_atoms, 3*nb_atoms))
 
-            Hdiag_icc = np.empty((nat, 3, 3))
+            else: 
+                H = bsr_matrix((H_pcc, j_p, first_i), shape=(3*nb_atoms, 3*nb_atoms))
+
+            Hdiag_icc = np.empty((nb_atoms, 3, 3))
             for x in range(3):
                 for y in range(3):
                     Hdiag_icc[:, x, y] = - \
-                        np.bincount(i_n, weights=H_ncc[:, x, y])
+                        np.bincount(i_p, weights=H_pcc[:, x, y])
+
+            if divide_by_masses:
+                H += bsr_matrix(((Hdiag_icc.T/masses_n).T, np.arange(nb_atoms),
+                             np.arange(nb_atoms+1)), shape=(3*nb_atoms, 3*nb_atoms))
+
+            else:
+                H += bsr_matrix((Hdiag_icc, np.arange(nb_atoms),
+                             np.arange(nb_atoms+1)), shape=(3*nb_atoms, 3*nb_atoms))
 
-            H += bsr_matrix((Hdiag_icc, np.arange(nat),
-                             np.arange(nat+1)), shape=(3*nat, 3*nat))
             return H
 
         # Dense matrix format
-        elif H_format == "dense":
-            e_nc = (dr_nc.T/abs_dr_n).T
-            H_ncc = -(dde_n * (e_nc.reshape(-1, 3, 1)
-                               * e_nc.reshape(-1, 1, 3)).T).T
-            H_ncc += -(de_n/abs_dr_n * (np.eye(3, dtype=e_nc.dtype)
-                                        - (e_nc.reshape(-1, 3, 1) * e_nc.reshape(-1, 1, 3))).T).T
+        elif format == "dense":
+            H = np.zeros((3*nb_atoms, 3*nb_atoms))
+            for atom in range(len(i_p)):
+                H[3*i_p[atom]:3*i_p[atom]+3,
+                  3*j_p[atom]:3*j_p[atom]+3] += H_pcc[atom]
 
-            H = np.zeros((3*nat, 3*nat))
-            for atom in range(len(i_n)):
-                H[3*i_n[atom]:3*i_n[atom]+3,
-                  3*j_n[atom]:3*j_n[atom]+3] += H_ncc[atom]
-
-            Hdiag_icc = np.empty((nat, 3, 3))
+            Hdiag_icc = np.empty((nb_atoms, 3, 3))
             for x in range(3):
                 for y in range(3):
                     Hdiag_icc[:, x, y] = - \
-                        np.bincount(i_n, weights=H_ncc[:, x, y])
+                        np.bincount(i_p, weights=H_pcc[:, x, y])
 
-            Hdiag_ncc = np.zeros((3*nat, 3*nat))
-            for atom in range(nat):
+            Hdiag_ncc = np.zeros((3*nb_atoms, 3*nb_atoms))
+            for atom in range(nb_atoms):
                 Hdiag_ncc[3*atom:3*atom+3,
                           3*atom:3*atom+3] += Hdiag_icc[atom]
 
             H += Hdiag_ncc
 
-            return H
+            if divide_by_masses:
+                masses_p = (self.atoms.get_masses()).repeat(3)
+                H /= np.sqrt(masses_p.reshape(-1,1)*masses_p.reshape(1,-1))
+                return H
+
+            else:
+                return H
+
+        # Neighbour list format
+        elif format == "neighbour-list":
+            return H_pcc, i_p, j_p, r_pc, r_p
