@@ -19,17 +19,13 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 # ======================================================================
 
-from __future__ import print_function
-
 import itertools
 import warnings
 
 import numpy as np
+import scipy.stats as scipy_stats
 from numpy.linalg import inv, norm
-try:
-    import scipy.stats as scipy_stats
-except:
-    scipy_stats = None
+from scipy.linalg import sqrtm
 
 import ase.units as units
 from ase.atoms import Atoms
@@ -51,9 +47,9 @@ def Voigt_6_to_full_3x3_strain(strain_vector):
     Form a 3x3 strain matrix from a 6 component vector in Voigt notation
     """
     e1, e2, e3, e4, e5, e6 = np.transpose(strain_vector)
-    return np.transpose([[1.0+e1, 0.5*e6, 0.5*e5],
-                         [0.5*e6, 1.0+e2, 0.5*e4],
-                         [0.5*e5, 0.5*e4, 1.0+e3]])
+    return np.transpose([[e1, 0.5*e6, 0.5*e5],
+                         [0.5*e6, e2, 0.5*e4],
+                         [0.5*e5, 0.5*e4, e3]])
 
 
 def Voigt_6_to_full_3x3_stress(stress_vector):
@@ -71,9 +67,9 @@ def full_3x3_to_Voigt_6_strain(strain_matrix):
     Form a 6 component strain vector in Voigt notation from a 3x3 matrix
     """
     strain_matrix = np.asarray(strain_matrix)
-    return np.transpose([strain_matrix[...,0,0] - 1.0,
-                         strain_matrix[...,1,1] - 1.0,
-                         strain_matrix[...,2,2] - 1.0,
+    return np.transpose([strain_matrix[...,0,0],
+                         strain_matrix[...,1,1],
+                         strain_matrix[...,2,2],
                          strain_matrix[...,1,2]+strain_matrix[...,2,1],
                          strain_matrix[...,0,2]+strain_matrix[...,2,0],
                          strain_matrix[...,0,1]+strain_matrix[...,1,0]])
@@ -117,14 +113,12 @@ def Voigt_6x6_to_full_3x3x3x3(C):
     return C_out
 
 
-def full_3x3x3x3_to_Voigt_6x6(C):
+def full_3x3x3x3_to_Voigt_6x6(C, tol=1e-3):
     """
     Convert from the full 3x3x3x3 representation of the stiffness matrix
     to the representation in Voigt notation. Checks symmetry in that process.
     """
 
-    tol = 1e-3
-
     C = np.asarray(C)
     Voigt = np.zeros((6,6))
     for i in range(6):
@@ -132,39 +126,39 @@ def full_3x3x3x3_to_Voigt_6x6(C):
             k, l = Voigt_notation[i]
             m, n = Voigt_notation[j]
             Voigt[i,j] = C[k,l,m,n]
-
-            #print '---'
-            #print k,l,m,n, C[k,l,m,n]
-            #print m,n,k,l, C[m,n,k,l]
-            #print l,k,m,n, C[l,k,m,n]
-            #print k,l,n,m, C[k,l,n,m]
-            #print m,n,l,k, C[m,n,l,k]
-            #print n,m,k,l, C[n,m,k,l]
-            #print l,k,n,m, C[l,k,n,m]
-            #print n,m,l,k, C[n,m,l,k]
-            #print '---'
-
+            """
+            print('---')
+            print("k,l,m,n", C[k,l,m,n])
+            print("m,n,k,l", C[m,n,k,l])
+            print("l,k,m,n", C[l,k,m,n])
+            print("k,l,n,m", C[k,l,n,m])
+            print("m,n,l,k", C[m,n,l,k])
+            print("n,m,k,l", C[n,m,k,l])
+            print("l,k,n,m", C[l,k,n,m])
+            print("n,m,l,k", C[n,m,l,k])
+            print('---')
+            """
             # Check symmetries
             assert abs(Voigt[i,j]-C[m,n,k,l]) < tol, \
-                'Voigt[{},{}] = {}, C[{},{},{},{}] = {}' \
+                '1 Voigt[{},{}] = {}, C[{},{},{},{}] = {}' \
                 .format(i, j, Voigt[i,j], m, n, k, l, C[m,n,k,l])
             assert abs(Voigt[i,j]-C[l,k,m,n]) < tol, \
-                'Voigt[{},{}] = {}, C[{},{},{},{}] = {}' \
-                .format(i, j, Voigt[i,j], k, l, m, n, C[l,k,m,n])
+                '2 Voigt[{},{}] = {}, C[{},{},{},{}] = {}' \
+                .format(i, j, Voigt[i,j], l, k, m, n, C[l,k,m,n])
             assert abs(Voigt[i,j]-C[k,l,n,m]) < tol, \
-                'Voigt[{},{}] = {}, C[{},{},{},{}] = {}' \
+                '3 Voigt[{},{}] = {}, C[{},{},{},{}] = {}' \
                 .format(i, j, Voigt[i,j], k, l, n, m, C[k,l,n,m])
             assert abs(Voigt[i,j]-C[m,n,l,k]) < tol, \
-                'Voigt[{},{}] = {}, C[{},{},{},{}] = {}' \
+                '4 Voigt[{},{}] = {}, C[{},{},{},{}] = {}' \
                 .format(i, j, Voigt[i,j], m, n, l, k, C[m,n,l,k])
             assert abs(Voigt[i,j]-C[n,m,k,l]) < tol, \
-                'Voigt[{},{}] = {}, C[{},{},{},{}] = {}' \
+                '5 Voigt[{},{}] = {}, C[{},{},{},{}] = {}' \
                 .format(i, j, Voigt[i,j], n, m, k, l, C[n,m,k,l])
             assert abs(Voigt[i,j]-C[l,k,n,m]) < tol, \
-                'Voigt[{},{}] = {}, C[{},{},{},{}] = {}' \
+                '6 Voigt[{},{}] = {}, C[{},{},{},{}] = {}' \
                 .format(i, j, Voigt[i,j], l, k, n, m, C[l,k,n,m])
             assert abs(Voigt[i,j]-C[n,m,l,k]) < tol, \
-                'Voigt[{},{}] = {}, C[{},{},{},{}] = {}' \
+                '7 Voigt[{},{}] = {}, C[{},{},{},{}] = {}' \
                 .format(i, j, Voigt[i,j], n, m, l, k, C[n,m,l,k])
 
     return Voigt
@@ -473,20 +467,28 @@ def measure_triclinic_elastic_constants(a, delta=0.001, optimizer=None,
             a.set_cell(cell, scale_atoms=True)
             a.set_positions(r0)
         
-            D = np.eye(3)
-            D[i, j] += 0.5*delta
-            D[j, i] += 0.5*delta
-            a.set_cell(np.dot(D, cell.T).T, scale_atoms=True)
+            e = np.zeros((3, 3))
+            e[i, j] += 0.5*delta
+            e[j, i] += 0.5*delta
+
+            F = np.eye(3) + e
+            #F = sqrtm(np.eye(3) + 2*e)
+            a.set_cell(np.dot(F, cell.T).T, scale_atoms=True)
             if optimizer is not None:
                 optimizer(a, logfile=logfile).run(**kwargs)
+            #sp = Voigt_6_to_full_3x3_stress(a.get_stress() * volume)
             sp = Voigt_6_to_full_3x3_stress(a.get_stress()*a.get_volume())
 
-            D = np.eye(3)
-            D[i, j] -= 0.5*delta
-            D[j, i] -= 0.5*delta
-            a.set_cell(np.dot(D, cell.T).T, scale_atoms=True)
+            e = np.zeros((3, 3))
+            e[i, j] -= 0.5*delta
+            e[j, i] -= 0.5*delta
+
+            F = np.eye(3) + e
+            #F = sqrtm(np.eye(3) + 2*e)
+            a.set_cell(np.dot(F, cell.T).T, scale_atoms=True)
             if optimizer is not None:
                 optimizer(a, logfile=logfile).run(**kwargs)
+            #sm = Voigt_6_to_full_3x3_stress(a.get_stress() * volume)
             sm = Voigt_6_to_full_3x3_stress(a.get_stress()*a.get_volume())
 
             C[:,:,i,j] = (sp-sm)/(2*delta*volume)
@@ -494,7 +496,7 @@ def measure_triclinic_elastic_constants(a, delta=0.001, optimizer=None,
     a.set_cell(cell, scale_atoms=True)
     a.set_positions(r0)
 
-    return full_3x3x3x3_to_Voigt_6x6(C)
+    return C
 
 
 
@@ -678,12 +680,12 @@ def generate_strained_configs(at0, symmetry='triclinic', N_steps=5, delta=1e-2):
         for step in range(N_steps):
             strain = np.where(pattern == 1, delta*(step+1-(N_steps+1)/2.0), 0.0)
             at = at0.copy()
-            if at0.get_calculator() is not None:
-                at.set_calculator(at0.get_calculator())
-            T = Voigt_6_to_full_3x3_strain(strain)
+            if at0.calc is not None:
+                at.calc = at0.calc
+            T = np.eye(3) + Voigt_6_to_full_3x3_strain(strain)
             at.set_cell(np.dot(T, at.cell.T).T, scale_atoms=False)
             at.positions[:] = np.dot(T, at.positions.T).T
-            at.info['strain'] = T
+            at.info['strain'] = T - np.eye(3)
             yield at
 
 
@@ -773,6 +775,9 @@ def fit_elastic_constants(a, symmetry='triclinic', N_steps=5, delta=1e-2, optimi
     """
 
     if isinstance(a, Atoms):
+        if a.constraints:
+            raise ValueError('Atoms passed to fit_elastic_constants() '
+                             'has constraints attached')
         # we've been passed a single Atoms object: use it to generate
         # set of strained configurations according to symmetry
         strained_configs = generate_strained_configs(a, symmetry, N_steps, delta)
