@@ -19,8 +19,6 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 # ======================================================================
 
-from __future__ import print_function
-
 import math
 import warnings
 import time
@@ -796,12 +794,12 @@ class SinclairCrack:
             u = u.reshape(self.N1, 3)
         offset = 3 * self.N1
         if self.variable_alpha:
-            alpha = x[offset]
+            alpha = float(x[offset])
             offset += 1
         else:
             alpha = defaults.get('alpha', self.alpha)
         if self.variable_k:
-            k = x[offset]
+            k = float(x[offset])
         else:
             k = defaults.get('k', self.k)
         return u, alpha, k
@@ -837,7 +835,8 @@ class SinclairCrack:
         u = np.c_[ux, uy, np.zeros_like(ux)] # convert to 3D field
         return u
 
-    def fit_cle(self, r_fit=20.0, variable_alpha=True, variable_k=True):
+    def fit_cle(self, r_fit=20.0, variable_alpha=True, variable_k=True, x0=None,
+                grid=None):
         def residuals(x, mask):
             idx = 0
             if variable_alpha:
@@ -855,15 +854,25 @@ class SinclairCrack:
             du = (self.k * self.u_cle(self.alpha) + u - k * self.u_cle(alpha))
             return du[mask, :].reshape(-1)
 
-        x0 = []
         mask = self.r < r_fit
-        if variable_alpha:
-            x0.append(self.alpha)
-        if variable_k:
-            x0.append(self.k)
-        res, ier = leastsq(residuals, x0, args=(mask,))
-        if ier not in [1, 2, 3, 4]:
-            raise RuntimeError('CLE fit failed')
+        if x0 is None:
+            x0 = []
+            if variable_alpha:
+                x0.append(self.alpha)
+            if variable_k:
+                x0.append(self.k)
+        if grid:
+            alpha_grid, k_grid = grid
+            vals = np.zeros((len(alpha_grid), len(k_grid)))
+            for i, alpha in enumerate(alpha_grid):
+                for j, k in enumerate(k_grid):
+                    vals[i, j] = (residuals([alpha, k], mask) ** 2).sum()
+            i_min, j_min = np.unravel_index(vals.argmin(), vals.shape)
+            return alpha_grid[i_min], k_grid[j_min]
+        else:
+            res, ier = leastsq(residuals, x0, args=(mask,))
+            if ier not in [1, 2, 3, 4]:
+                raise RuntimeError('CLE fit failed')
         return res
 
     def update_atoms(self):
@@ -888,7 +897,10 @@ class SinclairCrack:
 
     def set_atoms(self, atoms):
         N1_in = (atoms.arrays['region'] == 1).sum()
-        self.alpha = atoms.info['alpha']
+        if 'alpha' in atoms.info:
+            self.alpha = atoms.info['alpha']
+        else:
+            self.alpha = 0.0
         self.k = atoms.info['k']
         self.u[:] = np.zeros(3, self.N1)
         self.update_atoms()  # now we have same u_cle in atoms and self.atoms
@@ -901,8 +913,6 @@ class SinclairCrack:
         self.update_atoms()
 
     def get_crack_tip_force(self, forces=None, mask=None):
-        assert self.variable_alpha
-
         # V_alpha = -\nabla_1 U_CLE(alpha)
         tip_x = self.cryst.cell.diagonal()[0] / 2.0 + self.alpha
         tip_y = self.cryst.cell.diagonal()[1] / 2.0
@@ -2020,7 +2030,7 @@ def plot_stress_fields(atoms, r_range=None, initial_params=None, fix_params=None
     clf()
     for i, (ii, jj), label in zip(range(3),
                                   [(0,0), (1,1), (0,1)],
-                                  ['\sigma_{xx}', r'\sigma_{yy}', r'\sigma_{xy}']):
+                                  [r'\sigma_{xx}', r'\sigma_{yy}', r'\sigma_{xy}']):
         subplot(3,3,i+1)
         gca().set_aspect('equal')
         contourf(X, Y, grid_sigma[...,i]*GPA, contours[i])
