@@ -19,17 +19,16 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 # ======================================================================
 
-import itertools
-
 import numpy as np
 
 from ase.data import atomic_numbers
 
 import _matscipy
-from _matscipy import first_neighbours
+from _matscipy import first_neighbours, get_jump_indicies
 
 ###
 
+
 def mic(dr, cell, pbc=None):
     """
     Apply minimum image convention to an array of distance vectors.
@@ -54,7 +53,7 @@ def mic(dr, cell, pbc=None):
     # periodic boundaries then and need to be unwrapped.
     rec = np.linalg.inv(cell)
     if pbc is not None:
-        rec *= np.array(pbc, dtype=int).reshape(3,1)
+        rec *= np.array(pbc, dtype=int).reshape(3, 1)
     dri = np.round(np.dot(dr, rec))
 
     # Unwrap
@@ -69,7 +68,7 @@ def neighbour_list(quantities, atoms=None, cutoff=None, positions=None,
     are included in the neighbor list but the complexity of neighbor list search
     for those can become n^2.
 
-    The neighbor list is sorted by first atom index 'i', but not by second 
+    The neighbor list is sorted by first atom index 'i', but not by second
     atom index 'j'.
 
     The neighbour list accepts either an ASE Atoms object or positions and cell
@@ -126,7 +125,7 @@ def neighbour_list(quantities, atoms=None, cutoff=None, positions=None,
 
     2. Coordination counting with different cutoffs for each pair of species
         i = neighbor_list('i', a,
-                           {('H', 'H'): 1.1, ('C', 'H'): 1.3, ('C', 'C'): 1.85})
+                          {('H', 'H'): 1.1, ('C', 'H'): 1.3, ('C', 'C'): 1.85})
         coord = np.bincount(i)
 
     3. Pair distribution function:
@@ -187,7 +186,7 @@ e_nc = (dr_nc.T/abs_dr_n).T
     if atoms is None:
         if positions is None:
             raise ValueError('You provided neither an ASE Atoms object nor '
-                             'a positions array.') 
+                             'a positions array.')
         if cell is None:
             # Shrink wrapped cell
             rmin = np.min(positions, axis=0)
@@ -247,6 +246,62 @@ e_nc = (dr_nc.T/abs_dr_n).T
                                     _cutoff, numbers)
 
 
+def triplet_list(first_neighbours, abs_dr_p=None, cutoff=None, i_p=None, j_p=None):
+    """
+    Compute a triplet list for an atomic configuration. The triple list is a
+    mask that can be applied to the corresponding neighbour list to mask
+    triplet properties.
+    The triplet list accepts an first_neighbour array (generated by
+    first_neighbours) as input.
+
+    Parameters
+    ----------
+    first_neighbours : array
+        adresses of the first time an atom occours in the neighour list
+
+    Returns
+    -------
+    ij_t, ik_t : array
+        lists of adresses that form triples in the pair lists
+    jk_t : array (if and only if i_p, j_p, first_i != None)
+        list of pairs jk that connect each triplet ij, ik
+        between atom j and k
+
+    Example
+    -------
+    i_n, j_n, abs_dr_p = neighbour_list('ijd', atoms=atoms, cutoff=cutoff)
+
+    first_i = np.array([0, 2, 6, 10], dtype='int32')
+    a = triplet_list(first_i, [2.2]*9+[3.0], 2.6)
+
+    # one may obtain first_ij by using
+    find_ij = first_neighbours(len(i_p), ij_t)
+    # or (slower but less parameters and more general,
+    # i.e for every ordered list)
+    first_ij = get_jump_indicies(ij_t)
+
+    """
+    if not (abs_dr_p is None or cutoff is None):
+        res = _matscipy.triplet_list(first_neighbours, abs_dr_p, cutoff)
+    else:
+        res = _matscipy.triplet_list(first_neighbours)
+    # TODO: should be wrapped in c and be independet of i_n
+    # and j_n as of j_n is sorted; related issue #50
+    # add some tests!!!
+    if not (i_p is None or j_p is None or first_neighbours is None):
+        ij_t, ik_t = res
+        jk_t = -np.ones(len(ij_t), dtype='int32')
+        for t, (ij, ik) in enumerate(zip(ij_t, ik_t)):
+            for i in np.arange(first_neighbours[j_p[ij]],
+                               first_neighbours[j_p[ij]+1]):
+                if i_p[i] == j_p[ij] and j_p[i] == j_p[ik]:
+                    jk_t[t] = i
+                    break
+        return ij_t, ik_t, jk_t
+    else:
+        return res
+
+
 def find_indices_of_reversed_pairs(i_n, j_n, abs_dr_n):
     """Find neighbor list indices where reversed pairs are stored
 
@@ -254,7 +309,7 @@ def find_indices_of_reversed_pairs(i_n, j_n, abs_dr_n):
     determines the list of indices `reverse` into the neighbor list
     where each pair is reversed, i.e. `i_n[reverse[n]]=j_n[n]` and
     `j_n[reverse[n]]=i_n[n]` for each index `n` in the neighbor list
-    
+
     In the case of small periodic systems, one needs to be careful, because
     the same pair may appear more than one time, with different pair
     distances. Therefore, the pair distance must be taken into account.
@@ -278,10 +333,10 @@ def find_indices_of_reversed_pairs(i_n, j_n, abs_dr_n):
     """
     sorted_1 = np.lexsort(keys=(abs_dr_n, i_n, j_n))
     sorted_2 = np.lexsort(keys=(abs_dr_n, j_n, i_n))
-    #np.testing.assert_equal(j_n[sorted_1], i_n[sorted_2])
-    #np.testing.assert_equal(i_n[sorted_1], j_n[sorted_2])
-    #np.testing.assert_equal(abs_dr_n[sorted_1], abs_dr_n[sorted_2])
-    #print(np.c_[i_n[sorted_2], j_n[sorted_2], abs_dr_n[sorted_2], 
+    # np.testing.assert_equal(j_n[sorted_1], i_n[sorted_2])
+    # np.testing.assert_equal(i_n[sorted_1], j_n[sorted_2])
+    # np.testing.assert_equal(abs_dr_n[sorted_1], abs_dr_n[sorted_2])
+    # print(np.c_[i_n[sorted_2], j_n[sorted_2], abs_dr_n[sorted_2],
     #            i_n[sorted_1], j_n[sorted_1], abs_dr_n[sorted_1]])
     tmp2 = np.arange(i_n.size)[sorted_2]
     tmp1 = np.arange(i_n.size)[sorted_1]
